#ifndef CODEGEN_H
#define CODEGEN_H


// Load the Primary Register with a Constant
void LoadConstant(char* n);

// Load a Variable to the Primary Register
void LoadVariable(char* Name);

//Store the Primary Register to a Variable
void StoreVariable(char* Name);

// Negate Primary
void Negate();

// Push Primary to Stack
void Push();

// Add TOS to Primary
void PopAdd();

// Subtract TOS from Primary
void PopSub();

// Or TOS with Primary
void PopOr();

// Exclusive-Or TOS with Primary 
void PopXor();

// Multiply TOS by Primary
void PopMul();

// Divide Primary by TOS
void PopDiv();

// AND Primary with TOS
void PopAnd();

// Bitwise Not Primary
void NotIt();

// Call a subroutine
void CallSubroutine();

// Write Header Info
extern void Header();

// Write the Prolog
extern void Prolog();

// Write the Epilog
extern void Epilog();

// Allocate Storage for a Static Variable
void Allocate(char* name, char* val);

// Read Variable to Primary Register
void ReadIt();

// Write from Primary Register
void WriteIt();

// Branch False
void  BranchFalse(char* L);

// Branch Unconditional
void Branch(char* L);

// Post a Label To Output
void PostLabel(char* L);

// Set Primary If Compare was =
void SetEqual();

// Compare Top of Stack with Primary
void PopCompare();

// Set D0 If Compare was <=
void SetLessOrEqual();

// Set D0 If Compare was >=
void SetGreaterOrEqual();

// Set D0 If Compare was !=
void SetNEqual();

// Set D0 If Compare was <
void SetLess();

// Set Primary If Compare was >
void SetGreater();

// Clear the Primary Register
void Clear();

// Retrun from subroutine
void Return();

// Call a Procedure
void Call(char* Name);

// Load a Parameter to the Primary Register
void LoadParam(int N);

// Store a Parameter from the Primary Register
void StoreParam(int N);

// Adjust the Stack Pointer Upwards by N Bytes
void CleanStack(int N);

// Write the Prolog for a Procedure
void ProcProlog(char* L, int k);

// Write the Epilog for a Procedure
void ProcEpilog();


#endif
