			.clkmode xtal1 x8
			.xinfreq 10000000

                        org     0

enter
                        mov     dira, pins                 'Set Pins to output


                        mov     lmm_pc, lmm_program_addr   'Get address of function to call
                        add     lmm_pc, object_base        'Add this objects base address
                        jmp     #lmm_fetch                 'Start the LMM virtual machine

'This is a Large Memory Model (LMM) virtual machine
lmm_fetch               rdlong  :lmm_ins_0, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_0              nop
                        rdlong  :lmm_ins_1, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_1              nop
                        rdlong  :lmm_ins_2, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_2              nop
                        rdlong  :lmm_ins_3, lmm_pc
                        add     lmm_pc, #4
:lmm_ins_3              nop
                        jmp     #lmm_fetch

'LMM Jump
lmm_jmp                 rdlong  lmm_pc, lmm_pc
                        add     lmm_pc, object_base
                        jmp     #lmm_fetch

'LMM Jump Zero
lmm_jmp_z      if_z     jmp     #lmm_jmp
                        add     lmm_pc, #4
                        jmp     #lmm_fetch

'LMM Jump Not Zero
lmm_jmp_nz     if_nz    jmp     #lmm_jmp
                        add     lmm_pc, #4
                        jmp     #lmm_fetch

'LMM Jump Carry
lmm_jmp_c      if_c     jmp     #lmm_jmp
                        add     lmm_pc, #4
                        jmp     #lmm_fetch

'LMM Jump Not Carry
lmm_jmp_nc     if_nc    jmp     #lmm_jmp
                        add     lmm_pc, #4
                        jmp     #lmm_fetch

'LMM Call
lmm_call                rdlong  funcp, lmm_pc              'Read the call address
                        add     funcp, object_base
                        add     lmm_pc, #4
                        sub     SP, #4                     'Push the return address to stack
                        wrlong  lmm_pc, SP
                        mov     lmm_pc, funcp              'Set PC to call address
                        jmp     #lmm_fetch
'LMM Return
lmm_ret                 rdlong  lmm_pc, SP                 'Pop the return address
                        add     SP, #4
                        jmp     #lmm_fetch                 'and run with it

'LMM Halt
lmm_halt                jmp     #lmm_halt


lmm_load_pointer        rdlong  pointer, lmm_pc
                        add     pointer, object_base
                        add     lmm_pc, #4
                        jmp     #lmm_fetch

lmm_load_funcp          rdlong  funcp, lmm_pc
                        add     funcp, object_base
                        add     lmm_pc, #4
                        jmp     #lmm_fetch


'------------------ The following maths functions curtousy of ImageCrafts C compiler --------------------

temp                    long 0
tmp2                    long 0
ftemp                   long 0
ftemp2                  long 0

'div32s - Signed 32 bit division R0 = R0 / R1
'Divisor --> R1
'Dividend--> R0
'Result --> Quotient in R0
'Remainder in R1
div32s                  mov temp, #32
                        mov tmp2, #0                       ' tmp2 is temp register to hold the remainder
                        mov ftemp, R1   
                        shr ftemp, #31 wz                  ' get the divisor sign (ftemp)
              if_nz     sub tmp2, R1                       ' divisor = -divisor if < 0
              if_nz     mov R1, tmp2
                        mov ftemp2, R0  
                        shr ftemp2, #31 wz                 ' get the dividend sign (ftemp2)
                        mov tmp2, #0
              if_nz     sub tmp2, R0                       ' dividend = -dividend if < 0
              if_nz     mov R0, tmp2
                        mov tmp2, #0
                        xor ftemp, ftemp2                  ' sign of result (ftemp)
up
                        shl R0, #1 wc
                        rcl tmp2, #1 wc
                        cmp R1, tmp2 wc, wz
              if_a      jmp #down1                         ' Branch to down1
                        sub tmp2, R1
                        add R0, #1
down1
                        sub temp, #1 wz
              if_ne     jmp #up                            ' Branch to up
                        mov R1, tmp2
                        mov tmp2, #0
                        cmp ftemp2, #0 wz
              if_nz     sub tmp2, R1                       ' adjust the sign of remainder
              if_nz     mov R1, tmp2
                        mov tmp2, #0
                        cmp ftemp, #0 wz      
              if_nz     sub tmp2, R0
              if_nz     mov R0, tmp2                       ' adjust the sign of quotient
                        jmp #lmm_fetch

'div32u - Unsigned 32 bit division R0 = R0 / R1
'Divisor --> R1
'Dividend--> R0
'Result --> Quotient in R0
'Remainder in R1
div32u                  mov ftemp, #32
                        mov ftemp2, #0
up2                     shl r0, #1 wc
                        rcl ftemp2, #1 wc
                        cmp R1, ftemp2 wc, wz
              if_a      jmp #down                          'Branch to down
                        sub ftemp2, R1
                        add R0, #1
down                    sub ftemp, #1 wz
              if_ne     jmp #up2                           'Branch to up
                        mov R1, ftemp2
                        jmp #lmm_fetch       


'mul32 - multiplication
'R0 --> 1st operand (32 bit)
'R1 --> 2nd operand (32 bit)
'Result --> R0 (<= 32 bit)

mul32                   mov ftemp,#0
start                   cmp R0,#0 wz
              if_e      jmp #down3                         'Branch to down3
                        shr R0,#1 wc
              if_ae     jmp #down2                         'Branch to down2
                        add ftemp,R1 wc
down2                   shl R1,#1 wc
                        jmp #start                         'Branch to start
down3                   mov R0,ftemp
                        jmp #lmm_fetch

'------------------ The following maths function from the Paralax Spin interpreter --------------------
'----------------------------------------------------------------------------------



lmm_program_addr        long    @PROGRAM

pins                    long    $00FF0000                  'Propeller Demo Board LED pins

object_base             long    0                          'HUB address of this object 

lmm_pc                  long    0
SP                      long    0
SF                      long    0
lmm_bp                  long    0
funcp                   long    0
disp                    long    0
pointer                 long    0
save_pc                 long    0
R0                      long    0
R1                      long    0
R2                      long    0
R3                      long    0
R4                      long    0
R5                      long    0
R6                      long    0
R7                      long    0

D0                      long    0
D1                      long    0
A1                      long    0

minus_one               long    -1

