#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "scanner.h"
#include "input.h"
#include "output.h"
#include "error.h"


static char name[33];
static char number[33];

// Recognize an Alpha Character
int IsAlpha(char c)
{
	return (isalpha(c));
}


// Recognize a Numeric Character
int  IsDigit(char c)
{
	return (isdigit(c));
}

// Recognize an Alphanumeric Character
int IsAlnum(char c)
{
	return (isalnum(c));
}

// Recognize an Addition Operator
int IsAddop(char c)
{
	return ((c == '+') || (c == '-') || (c == '|') || (c == '~'));
}

// Recognize a Multiplication Operator }
int IsMulop(char c)
{
	return ((c == '*') || (c == '/') || (c == '&'));
}


//Skip A Comment Field, recursive comments allowed
void SkipComment()
{
	while (Look != '}')
	{
		GetChar();
		if (Look == '{')
		{
			SkipComment();
		}
	}
	GetChar();
}


// Recognize White Space
int IsWhite(char c)
{
	return ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r')  || (c == '{'));
}

// Skip Over Leading White Space
void SkipWhite()
{
	while (IsWhite(Look))
	{
		if (Look == '{')
		{
			SkipComment();
		}
		else
		{
      			GetChar();
		}
	}
}

// FIXME THIS IS NOT USED BY TINY
// Match One Character
void Match(char c)
{
	char expstr[2];

	memset (expstr, 0, 2);
	if (Look == c)
	{
		GetChar();
		SkipWhite();
	}
	else
	{
		expstr[0] = c;
		Expected(expstr);
	}
}




// Get an Identifier
// DECISION: Identifiers are case sensitive.
void GetName()
{
	int i;

	SkipWhite();
	if (IsAlpha(Look) == 0)
	{
		Expected("Name");
	}
	i = 0;
	memset(Value, 0, 33);
	Token = 'x';
	while (IsAlnum(Look))
	{
		if (i < 32)
		{
			Value[i++] = Look;
		}
		GetChar();
	}
}

// Get a Number 
// DECISION: This returns a string not an integer
#ifdef numbers_as_integers 
//int GetNumber()
#else
char* GetNumber()
#endif
{
	int i;

	i = 0;
	memset(name, 0, 33);
	if (IsDigit(Look) == 0)
	{
		Expected("Integer");
	}
	while (IsDigit(Look))
	{
		if (i < 32)
		{
			number[i++] = Look;
		}
		GetChar();
	}
	SkipWhite();
#ifdef numbers_as_integers 
	return (atoi(number));
#else
	return (number);
#endif
}


// NEW FOR TINY STARTS HERE //////////////////////////////////////////////7

char Token;
char Value[33];

// Get a Number 
void GetNum()
{
	int i;

	SkipWhite();
   	if (IsDigit(Look) == 0)
	{
		Expected("Number");
	}
	Token = '#';
	memset(Value, 0, 33);
	i = 0;
	while (IsDigit(Look))
	{
		if (i < 32)
		{
			Value[i++] = Look;
		}
		GetChar();
	}
}

// Get an Operator
void GetOp()
{
	SkipWhite();
	Token = Look;
	memset(Value, 0, 33);
	Value[0] = Look;
	GetChar();
}



// Get the Next Input Token
void Next()
{
	SkipWhite();
	if (IsAlpha(Look))
	{
		GetName();
	}
	else if (IsDigit(Look))
	{
		GetNum();
	}
	else
	{
		GetOp();
	}
}



// Match a Specific Input String
void MatchString(char* x)
{
	if (strcmp (x, Value) != 0)
	{
		printf ("Expected: %s got %s\n", x, Value);
	}
	else
	{
		Next();
	}
}

// Match a Semicolon
void Semi()
{
#ifdef MANDATORY_SEMICOLONS
	MatchString(";");
#else
	if (Token == ';')
	{
		Next();
	}
#endif
}

#define NKW  12
#define NKW1 13


static char* KWlist[NKW]  = {"IF", "ELSE", "ENDIF", "WHILE", "ENDWHILE", "READ", "WRITE", "VAR", "END", "PROGRAM", "PROCEDURE", "FUNCTION"};

static char  KWcode[NKW1] = {'x', 'i', 'l', 'e', 'w', 'e', 'R', 'W', 'v', 'e', 'P', 'p', 'F'}; // N.B. 'f' is used elsewhere to mean formal param


// Scan the Current Identifier for Keywords
void Scan()
{
	int i;

	if (Token == 'x')
	{
		for (i = 0; i < NKW; i++)
		{
			if (strcmp(Value, KWlist[i]) == 0)
			{
				Token = KWcode[i + 1];
			}
		}
	}
}

// Recognize a Relop
int IsRelop(char c)
{
 	return ((c == '=') || (c == '#') || (c == '<') || (c == '>'));
}

// Recognize a Boolean Orop
int IsOrOp(char c)
{
 	return ((c == '|') || (c == '~'));
}
